/***************************************************************************************/
/* Elektronischer Adventkalender in C                                                  */
/*                                                                                     */
/* externer Takt (20 MHz)                                                              */
/*                                                                                     */
/* Compiler: CC5X                                                                      */
/*                                                                                     */
/* Entwickler: Buchgeher Stefan                                                        */
/* Entwicklungsbeginn der Software: 27. September 2008                                 */
/* Funktionsfaehig seit: 27. September 2008                                            */
/* Letzte Bearbeitung: 21. Juni 2009                                                   */
/***************************************************************************************/

/****************** Include-Dateien ****************************************************/
#include "PROJEKT.H"
#include <int16CXX.H>

#pragma sharedAllocation


/****************** Strukturen *********************************************************/
/* keine Strukturen verwendet */


/****************** Externe Register ***************************************************/
char	Flackerlichter0;
char	Flackerlichter1;
char	Flackerlichter2;
char	Flackerlichter3;
char	Flackerlichter4;
char	Flackerlichter5;
char	Flackerlichter6;
char	Flackerlichter7;
char	Flackerlichter8;
char	Flackerlichter9;
char	Flackerlichter10;
char	Flackerlichter11;
char	Flackerlichter12;

char    TagesZaehler;


/****************** Bits in den externen Registern *************************************/
/* keine Bits in den externen Registern */


/****************** Globale Register ***************************************************/
char    FlagISRHP;
char    ZaehlerZeitbasis10ms;
char    ZaehlerZeitbasis1sek;
char    ZaehlerZeitbasis1min;
char    ZaehlerZeitbasis1std;
char    ZaehlerZeitbasis1tag;

char    TasteAlt;               // Hilfsregister zur Erkennung ob die Taste
char    TasteStatus;            // gedrueckt wurde

char    TagesZaehlerEnable;     // Hilfsregister: Die automatische Tageszaehlung soll
                                // erst nach dem ersten Tastendruck erfolgen

    
/****************** Bits in den globalen Registern *************************************/
/* Register FlagISRHP */
bit FlagZeitbasis409us  @    FlagISRHP.0;
bit FlagZeitbasis10ms   @    FlagISRHP.1;
bit FlagZeitbasis1tag   @    FlagISRHP.2;

/* Register TasteStatus */
bit Taster_Status       @    TasteStatus.0;


/****************** Funktionsprototypen ************************************************/
/* Unterprogramm zur Initialisierung des Mikroccntrollers */
void Init(void);

/* Unterprogramm zur Ermittlung ob, und welche Taste gedrueckt wurde */
void TastenRoutine(void);

/* Unterprogramm zum Zaehlen der Tage */
void Naechster_Tag(void);

/* Unterprogramme zur Kommunikation mit dem Schieberegister 74xx595 */
void SR_74xx595_SendData(void);
void SR_74xx595_SendByte(char Wert);


/****************** Konfigurations-Bits ************************************************/ 
#pragma config1 |= 0b.10111101101010
/*                    +---------------- Bit 13 (CP): Flash Memory Code Protection bits
                       |||||||||||||         0 : 0000 to 0FFFh protected (All proteced)
                       |||||||||||||    ->   1 : Code protection off
                       +--------------- Bit 12 (CCPMX): CCP1 Pin Selection bit 
                        ||||||||||||    ->   0 : CCP1 function on RB3
                        ||||||||||||         1 : CCP1 function on RB0
                        +-------------- Bit 11 (DEBUG): In-Circuit Debugger Mode
                         |||||||||||         0 : DEBUG on  (= enabled)
                         |||||||||||    ->   1 : DEBUG off (= disabled)
                         ++------------ Bit 10-9 (WRT1:WRT0): FLASH Memory Write Enable
                           |||||||||       0 0 : 0000h to 0FFFh write protected
                           |||||||||       0 1 : 0000h to 07FFh write protected
                           |||||||||       1 0 : 0000h to 00FFh write protected
                           |||||||||    -> 1 1 : WRT off   (= diasbled)
                           +----------- Bit  8 (CPD): Data EE Memory Code Protection
                            ||||||||         0 : CPD on    (= enabled)
                            ||||||||    ->   1 : CPD off   (= disabled)
                            +---------- Bit  7 (LVP): Low Voltage ICSP Enable Bit
                             |||||||    ->   0 : LVP off   (= disabled)
                             |||||||         1 : LVP on    (= enabled)
                             +--------- Bit  6 (BOREN): Brown-Out Reset Enable bit
                              ||||||         0 : BOR enabled
                              ||||||    ->   1 : BOR disabled
                              +-------- Bit 5 (MCLRE): RA5/nMCLR Pin Function Sel. bit
                               |||||         0 : RA5 is Digital I/O
                               |||||    ->   1 : RA5 is nMCLR
                               |+------ Bit  3 (nPWRTEN): Power-Up Timer Enable bit
                               | |||         0 : PWRT on   (= enabled)
                               | |||    ->   1 : PWRT off  (= disabled)
                               | +----- Bit  2 (WDTE): Watchdog Timer
                               |  ||    ->   0 : WDT off   (= disabled)
                               |  ||         1 : WDT on    (= enabled))
                               +--++--- Bit 4,1-0 (FOSC2:FOSC0): Oszillator Selection
					   000 : LP Oszillator    (RA6=CLKOUT, RA7=CLKIN)
	                                   001 : XT Oszillator    (RA6=CLKOUT, RA7=CLKIN)
                                        -> 010 : HS Oszillator    (RA6=CLKOUT, RA6=CLKIN)
					   011 : EXTCLK Oszillator (RA6=I/O, RA7=CLKIN)
					   100 : INTRC Oszillator (RA6=I/O, RA7=I/O)
					   101 : INTRC Oszillator (RA6=CLKOUT, RA7=I/O)
					   110 : EXTRC Oszillator (RA6=I/O, RA7=CLKIN)
				           111 : EXTRC Oszillator (RA6=CLKOUT, RA7=CLKIN)
*/

#pragma config2 |= 0b.11111111111100
/*                    ++++++++++++----- Bit 13-2 : Reserve
                                  +|--- Bit 1 (IESO) Einternal External Switch Over bit   
                                   |    ->   0 : IESO disabled    
                                   |         1 : IESO enabled
                                   +--- Bit 0 (FCMEN): Fail-Safe Clock Monitor Enable bit
                                        ->   0 : FCMEN disabled 
                                             1 : FCMEN enabled 
*/


/****************** ISR - Timer0 *******************************************************/

/***************************************************************************************/
/* Interrupt Service Routine:                                                          */
/*                                                                                     */
/* Aufruf:                                                                             */
/*      alle 409,6 us                                                                  */
/*                                                                                     */
/* Aufgaben:                                                                           */
/*      + Zeitbasen fuer 409 Mikrosekunde, 10 Millisekunde und 1 Tag erzeugen          */
/*      + Das Timer-Interrupt-Flag T0IF wieder loeschen                                */
/***************************************************************************************/
#pragma origin 4

interrupt InterruptRoutine(void)       // Interruptroutine
{
  int_save_registers

   FlagZeitbasis409us = 1;             // Botschaftsflag fuer 409us-Zeitbasis setzen

    ZaehlerZeitbasis10ms--;
    if (ZaehlerZeitbasis10ms == 0)     // Botschaftsflag fuer 10ms-Zeitbasis setzen
    {
        FlagZeitbasis10ms = 1;
        ZaehlerZeitbasis10ms = KONSTZEITBASIS10MS; // Zaehlregister fuer 10-ms-Zeitbasis
                                       // neu laden
        ZaehlerZeitbasis1sek--;
        if (ZaehlerZeitbasis1sek == 0)
        {
            ZaehlerZeitbasis1sek = KONSTZEITBASIS1SEK; // Zaehlregister fuer 1-Sekunde-
                                       // Zeitbasis neu laden
            ZaehlerZeitbasis1min--;
            if (ZaehlerZeitbasis1min == 0)
            {
                ZaehlerZeitbasis1min = KONSTZEITBASIS1MIN; // Zaehlregister fuer
                                       //1-Minuten-Zeitbasis neu laden
                ZaehlerZeitbasis1std--;
                if (ZaehlerZeitbasis1std == 0)
                {
                    ZaehlerZeitbasis1std = KONSTZEITBASIS1STD; // Zaehlregister fuer
                                       // 1-Stunden-Zeitbasis neu laden
                    ZaehlerZeitbasis1tag--;
                    if (ZaehlerZeitbasis1tag == 0)
                    {
                        FlagZeitbasis1tag = 1;  // Botschaftsflag fuer
                                       // 1-Tages-Zeitbasis setzen
                        ZaehlerZeitbasis1tag = KONSTZEITBASIS1TAG; // Zaehlregister
                                       // fuer 1-Tages-Zeitbasis neu laden
                    }
                }
            }
        }
    }

    T0IF = 0;                   // Timer-Interrupt-Flag T0IF wieder loeschen

    int_restore_registers
}


/***************************************************************************************/
/* Init:                                                                               */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   Initialisierung des Prozessor:                                                    */
/*      + Timer 0 (TMR0) loeschen                                                      */
/*      + Timer 0-ISR soll ca. alle 409us aufgerufen werden, daher den Vorteiler mit   */
/*        1:8 laden (bei einem internen 20-MHz-Takt)                                   */
/*      + Wichtig: Port A auf digitale I/O-Pins einstellen, da der PIC-interne ADC     */
/*        hier nicht verwendet wird!                                                   */
/*      + diverse Register initialisieren                                              */
/*          + Zaehlregister fuer Zeitbasen initialisieren                              */
/*          + Tageszaehler loeschen                                                    */
/*          + Register fuer die Tastenabfrage initialisieren                           */
/*      + Interrupt freigeben (Timer0 freigeben durch Setzen von GIE und T0IE im       */
/*        INTCON-Register)                                                             */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   keiner                                                                            */
/***************************************************************************************/
void Init(void)
{
    // Timer-0-Interrupt
    TMR0 = 0;                        // Timer 0 auf 0 voreinstellen
 
    OPTION_REG = 0b.1000.0010;       // Option-Register (Bank 1)
/*                  +--------------- Bit 7 (nRBPU): Pull-Up-Widerstaende am Port B
                     ||| ||||             0 : Pull-Up aktiviert
                     ||| ||||        ->   1 : Pull-Up deaktiviert
                     +-------------- Bit 6 (INTEDG): Interrupt Edge Select Bit
                      || ||||        ->   0 : Interrupt on falling edge of RB0/INT pin
                      || ||||             1 : Interrupt on rising edge of RB0/INT pin
                      +------------- Bit 5 (T0CS): Timer 0 Clock source Select Bit
                       | ||||        ->   0 : Transition on RA4/T0CKI pin
                       | ||||             1 : Internal instruction cycle clock (CLKOUT)
                       +------------ Bit 4 (T0SE): TMR0 Source Edge Select bit
                         ||||        ->   0 : Increment on low-to-high tranition
                         ||||             1 : Increment on high-to-low tranition
                         +---------- Bit 3 (PSA): Prescaler Assignment bit
                          |||        ->   0 : Prescaler is assigned to the TIMER0 module
                          |||             1 : Prescaler is assigned to the WDT
                          +++--------- Bit 2-0 (PS2:PS0): Prescaler Rate Select bits
                                       Bit Value        TMR0 Rate         WDT Rate
                                         000 :             1:2               1:1
                                         001 :             1:4               1:2
                                     ->  010 :             1:8               1:4
                                         011 :             1:16              1:8
                                         100 :             1:32              1:16
                                         101 :             1:64              1:32
                                         110 :             1:128             1:64
                                         111 :             1:256             1:128
*/

    // Ports konfigurieren
    TRISA = 0b.1110.1001;            // Richtungsregister Port A (0: Ausgang, 1: Eingang)
/*             +-------------------- Bit 7 Port RA7: hier unbenutzt
                +------------------- Bit 6 Port RA6: hier unbenutzt
                 +------------------ Bit 5 Port RA5: hier unbenutzt
                  +----------------- Bit 4 Port RA4: hier Ausgang (Steuerleitung TAKT)
                    +--------------- Bit 3 Port RA3: hier unbenutzt
                     +-------------- Bit 2 Port RA2: hier Ausgang (Steuerleitung DATA)
                      +------------- Bit 1 Port RA1: hier Ausgang (Steuerleitung LOAD)
                       +------------ Bit 0 Port RA0: hier unbenutzt
*/

    TRISB = 0b.0000.1111;            // Richtungsregister Port B (0: Ausgang, 1: Eingang)
/*             +-------------------- Bit 7 Port RB7: hier unbenutzt
                +------------------- Bit 6 Port RB6: hier unbenutzt
                 +------------------ Bit 5 Port RB5: hier unbenutzt
                  +----------------- Bit 4 Port RB4: hier unbenutzt
                    +--------------- Bit 3 Port RB3: hier Eingang (Jumper 3)
                     +-------------- Bit 2 Port RB2: hier Eingang (Jumper 2)
                      +------------- Bit 1 Port RB1: hier Eingang (Jumper 1)
                       +------------ Bit 0 Port RB0: hier Eingang (Taster)
*/

    // Comparator-Eingaenge deaktivieren
    PORTA = 0;
    CMCON = 7;


    // Alle Kerzen und den Baum und den Stern deaktivieren (ausschalten)
    Flackerlichter0 = 0;
    Flackerlichter1 = 0;
    Flackerlichter2 = 0;
    Flackerlichter3 = 0;
    Flackerlichter4 = 0;
    Flackerlichter5 = 0;
    Flackerlichter6 = 0;
    Flackerlichter7 = 0;
    Flackerlichter8 = 0;
    Flackerlichter9 = 0;
    Flackerlichter10 = 0;
    Flackerlichter11 = 0;
    Flackerlichter12 = 0;
    SR_74xx595_SendData();
    

    // Zaehlregister fuer Zeitbasen initialisieren
    ZaehlerZeitbasis10ms = KONSTZEITBASIS10MS; // Zaehlregister fuer 10-ms-Zeitbasis
                                               // initialisieren
    ZaehlerZeitbasis1sek = KONSTZEITBASIS1SEK; // Zaehlregister fuer 1-Sekunden-Zeitbasis
                                               // initialisieren
    ZaehlerZeitbasis1min = KONSTZEITBASIS1MIN; // Zaehlregister fuer 1-Minuten-Zeitbasis
                                               // initialisieren
    ZaehlerZeitbasis1std = KONSTZEITBASIS1STD; // Zaehlregister fuer 1-Stunden-Zeitbasis
                                               // initialisieren
    ZaehlerZeitbasis1tag = KONSTZEITBASIS1TAG; // Zaehlregister fuer 1-Tages-Zeitbasis
                                               // initialisieren

    // Tageszaehler loeschen
    TagesZaehler = 0;

    // Register fuer die Tastenabfrage initialisieren
    TasteStatus = 0;
    TasteAlt = PORTB;

    TagesZaehlerEnable = 0;

    // Interrupt freigeben
    INTCON = 0b.1010.0000;           // Interrupt-Control-Register (Bank 1)
/*              +------------------- Bit 7 (GIE): Global Interrupt Enable bit
                 ||| ||||                 0 : Disables all interrupts
                 ||| ||||            ->   1 : Enables all unmasked interrupts
                 +------------------ Bit 6 (PEIE): Peripheral Interrupt Enable bit
                  || ||||            ->   0 : Disables all peripheral interrupts
                  || ||||                 1 : Enables all unmasked peripheral interrupts
                  +----------------- Bit 5 (T0IE): Timer 0 Overflow Interrupt Enable Bit
                   | ||||                 0 : Disabled
                   | ||||            ->   1 : Enabled
                   +---------------- Bit 4 (INTE): RB0/INT External Interrupt Enable Bit
                     ||||            ->   0 : Disabled
                     ||||                 1 : Enabled
                     +-------------- Bit 3 (RPIE): RB Port Change Interrupt Enable Bit
                      |||            ->   0 : Disabled
                      |||                 1 : Enabled
                      +------------- Bit 2 (T0IF): Timer 0 Overflow Interrupt Flag bit
                       ||            ->   0 : TMR0 register did not overflow
                       ||                 1 : TMR0 register has overflowed
                       +------------ Bit 1 (INTF): RB0/INT External Interrupt Flag bit
                        |            ->   0 : RB0/INT external interrupt did not occur
                        |                 1 : RB0/INT external interrupt occurred
                        +----------- Bit 0 (RBIF): RB Port Change Interrupt Flag bit
                                     ->   0 : None of RB7:RB4 pins have changed state
                                          1 : One of the RB7:RB4 pins changed state
*/
}


/***************************************************************************************/
/* TastenRoutine:                                                                      */
/*                                                                                     */
/* Aufruf:                                                                             */
/*   alle 10 ms (vom Hauptprogramm)                                                    */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   Ermitteln, ob die Taste gedrueckt wurde, und das Register TagesZaehler erhoehen,  */
/*   oder loeschen, wenn das Register TagesZaehler den Wert 25 (oder mehr) besitzt.    */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*   + Port B einlesen und im Register temp1 sichern                                   */
/*   + dieses Register invertieren und in temp2 sichern (temp1 wird spaeter noch       */
/*     einmal benoetigt und darf daher nicht ueberschrieben werden!)                   */
/*   + Das Register TasteAlt gibt beim Aufruf (dieses Unterprogramms) den Zustand der  */
/*     Tasten an, welcher beim vorhergehenden Aufruf dieses Unterprogramms am Port B   */
/*     herrschte. Also den Zustand am Port B vor 10 ms.                                */
/*   + Mit Hilfe der beiden Register temp2 und TasteAlt laesst sich somit ermitteln,   */
/*     ob eine bestimmte Taste zwischen dem vorhergehenden Aufruf dieses Unter-        */
/*     programms und den gerade bearbeitenden Aufruf gedrueckt wurde. Dies erfolgt mit */
/*     einer bitweisen logischen-UND-Verknuepfung. Das Ergebnis dieser UND-Verknuepfung*/
/*     im Register TasteStatus sichern.                                                */
/*     Mit diesem Verfahren wird auch gleichzeitig das sogenannte Tastenprellen ueber- */
/*     brueckt, da dieses Tastenprellen in der Regel weit weniger als 10ms dauert.     */
/*   + Je nachdem welche Taste nun gedrueckt wurde, das entsprechende Bit im Register  */
/*     TasteStatus ist gesetzt, diese Kerze ein bzw. ausschalten (also invertieren).   */
/*   + Register TasteAlt mit dem Inhalt von temp1 fuer den naechsten Unterprogramm-    */
/*     aufruf laden.                                                                   */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Anmerkungen:                                                                        */
/*   + Die Tasten sind am Port B mit je einem Pull-up-Widerstand angeschlossen. Im     */
/*     Ruhezustand (Taste nicht gedrueckt) liegt an dem Portpin ein High-Pegel. Durch  */
/*     Druecken der Taste wird dieser Portpin auf Masse gelegt (Low-Pegel). Dieses     */
/*     Verhalten wird auch als low-aktiv bezeichnet.                                   */
/*   + Das so genannte Tastenprellen wird hier umgangen (Die Prellzeit ist kleiner als */
/*     das Aufrufintervall von 10ms)                                                   */
/***************************************************************************************/
void TastenRoutine(void)
{
    char temp1;
    char temp2;

    temp1 = PORTB;

    temp2 = ~temp1;                 // Alle Bits invertieren

    TasteStatus = TasteAlt & temp2;

    if (Taster_Status)
    {
        TagesZaehler++;
        if (TagesZaehler > 24)
        {
            TagesZaehler = 0;
        }

        ZaehlerZeitbasis1min = KONSTZEITBASIS1MIN; // Zaehlregister fuer
                                               // 1-Minuten-Zeitbasis initialisieren
        ZaehlerZeitbasis1std = KONSTZEITBASIS1STD; // Zaehlregister fuer
                                               // 1-Stunden-Zeitbasis initialisieren
        ZaehlerZeitbasis1tag = KONSTZEITBASIS1TAG; // Zaehlregister fuer
                                               // 1-Tages-Zeitbasis initialisieren

        TagesZaehlerEnable = 1;                // Automatische Tageszaehlung freigeben
    }

    TasteAlt = temp1;
}


/***************************************************************************************/
/* Naechster_Tag:                                                                      */
/*                                                                                     */
/* Aufruf:                                                                             */
/*   alle 24 Stunden (vom Hauptprogramm)                                               */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   Falls TagesZaehlerEnable gesetzt und der Tageszaehler noch kleiner als 24 ist,    */
/*   den Tageszaehler erhoehen:                                                        */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Anmerkung:                                                                          */
/*   besitzt das Zaehlregister (hier TagesZaehler) den Wert 24, so wird dieser Zaehler */
/*   nicht mehr erhoeht. D.h. Der Baum, alle Kerzen und der Stern "blinken" weiter.    */
/*   Das "loeschen" ist nur mit dem Taster oder mit dem Ein/Ausschalter meglich!       */
/***************************************************************************************/
void Naechster_Tag(void)
{
    if (TagesZaehlerEnable)
    {
        if (TagesZaehler < 24)
        {
            TagesZaehler++;
        }
    }
}


/***************************************************************************************/
/* SR_74xx595_SendByte:                                                                */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   Den seriellen Datenstrom mit Hilfe der hintereinandergeschalteten Schieberegister */
/*   vom Typ 74xx595 ausgeben.                                                         */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*   Mit Hilfe einer Schleife das MSB (Bit 7) am Datenausgang ausgeben, einen Takt     */
/*   erzeugen und alle Bits um eine Stelle weiterschieben                              */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   den zu "schiebende" Wert                                                          */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   keiner                                                                            */
/*                                                                                     */
/* Anmerkung:                                                                          */
/*   Dies ist ein Hilfsprogramm und darf nur vom Unterprogramm SR_74xx595_SendData     */
/*   aufgerufen werden. Daher befindet sich der zugehoerige Funktionsprototyp nicht in */
/*   der Header-Datei SR_74XX595.H sondern in der Quelldatei SR_74XX595.C. Wird dieses */
/*   Unterprogramm in einem anderen Unterprogramm oder im Hauptprogramm aufgerufen, so */
/*   der Compiler einen Fehler, oder zumindest eine Warnung ausgeben!                  */
/***************************************************************************************/
void SR_74xx595_SendByte(char Wert)
{
    char i;


    // MSB am Datenausgang ausgeben
    for (i=8; i>0; i--)
    {
        Port_DataIn = Wert.7;

        // Takt erteugen
        Port_Clock = 1;
	Port_Clock = 0;

        // Alle Bits um eine Stelle weiterschieben
	Wert = Wert << 1;
    }
}


/***************************************************************************************/
/* SR_74xx595_SendData:                                                                */
/*                                                                                     */
/* Aufgabe:                                                                            */
/*   Den seriellen Datenstrom mit Hilfe der hintereinandergeschalteten Schieberegister */
/*   vom Typ 74xx595 ausgeben.                                                         */
/*                                                                                     */
/* Vorgehensweise:                                                                     */
/*   + Die Steuerleitung Port_Enable loeschen                                          */
/*   + Nacheinander die Datenbits (in den Registern FlackerlichterXX) mit dem Unter-   */
/*     programm SR_74xx595_SendByte in die Schieberegister schieben                    */
/*   + Nach dem letzen Datenbit einen Ladeimpuls fuer die Uebernahme in die Schiebe-   */
/*     register erzeugen                                                               */
/*                                                                                     */
/* Uebergabeparameter:                                                                 */
/*   keine                                                                             */
/*                                                                                     */
/* Rueckgabeparameter:                                                                 */
/*   keiner                                                                            */
/***************************************************************************************/
void SR_74xx595_SendData(void)
{
    // Steuerleitung Port_Enable loeschen
    Port_Enable = 0;

    // Nacheinander die Datenbits (in den Registern FlackerlichterXX) mit dem 
    // Unterprogramm SR_74xx595_SendByte in die Schieberegister schieben
    SR_74xx595_SendByte(Flackerlichter12);
    SR_74xx595_SendByte(Flackerlichter11);
    SR_74xx595_SendByte(Flackerlichter10);
    SR_74xx595_SendByte(Flackerlichter9);
    SR_74xx595_SendByte(Flackerlichter8);
    SR_74xx595_SendByte(Flackerlichter7);
    SR_74xx595_SendByte(Flackerlichter6);
    SR_74xx595_SendByte(Flackerlichter5);
    SR_74xx595_SendByte(Flackerlichter4);
    SR_74xx595_SendByte(Flackerlichter3);
    SR_74xx595_SendByte(Flackerlichter2);
    SR_74xx595_SendByte(Flackerlichter1);
    SR_74xx595_SendByte(Flackerlichter0);

    // Ladeimpuls fuer die Uebernahme in die Schieberegister
    Port_Enable = 1;
    Port_Enable = 0;
}

 
/****************** Hauptprogramm ******************************************************/

/***************************************************************************************/
/* Aufgaben des Hauptprogramms:                                                        */
/*   + Controller initialisieren (Unterprogramm Init)                                  */
/*   + Zaehlregister fuer die Flackerlichter initialisieren (Unterprogramme            */
/*     Init_Flackerlicht_Baum, Init_Flackerlicht_Kerzen und Init_Flackerlicht_Stern)   */
/*   + Taetigkeiten/Unterprogramme, die alle zyklisch durchgefuehrt werden muessen:    */
/*      + alle 409 us: Routinen fuer die Flackerlichter ausfuehren, und den Zustand    */
/*        der LEDs mit Hilfe der Schieberegister ausgeben                              */
/*      + alle 10 ms: Tasten abfragen (Unterprogramm TastenRoutine)                    */
/*      + alle 24 Stunden (also jeden Tag): Tageszaehler erhoehen (Unterprogramm       */
/*        Naechster_Tag)                                                               */
/*                                                                                     */
/* Anmerkung:                                                                          */
/*   Nach der Ausfhrung der durch die gesetzten Botschaftsflags ausgeloesten Aufgaben */
/*   muss das Botschaftsflag geloescht werden. Wrde das Botschaftsflag nicht geloescht*/
/*   werden, so wuerde diese Aufgabe unuterbrochen wiederholt werden!                  */
/***************************************************************************************/
void main(void)
{
    Init();                            // Controller initialisieren
    Init_Flackerlicht_Baum();          // Register fuer das Flackerlicht initialisiern
    Init_Flackerlicht_Kerzen();
    Init_Flackerlicht_Stern();

    while(1)                           // Endlosschleife
    {
       if (FlagZeitbasis409us)
        {
            Flackerlicht_Baum();
            Flackerlicht_Kerzen();
            Flackerlicht_Stern();

            SR_74xx595_SendData();

            FlagZeitbasis409us = 0;     // Botschaftsflag (fuer 409us-Zeitbasis)
        }                               // zuruecksetzen

        if (FlagZeitbasis10ms)
        {
            TastenRoutine();
            FlagZeitbasis10ms = 0;      // Botschaftsflag (fuer 10ms-Zeitbasis)
        }                               // zuruecksetzen

        if (FlagZeitbasis1tag)
        {
            Naechster_Tag();
            FlagZeitbasis1tag = 0;      // Botschaftsflag (fuer 1-Tag-Zeitbasis)
        }                               // zuruecksetzen
    }
}